## How to fuzz

First of all, clone the repo, and execute `git submodule update --init` to load third party libs
### Easy way

Slower fuzzing, but you'll only need docker:

```
docker build -f fuzzing/Dockerfile -t fuzzer .
docker run -v $(pwd)/fuzzing:/libddwaf/fuzzing fuzzer ./fuzzing/run.sh
```

### Hard way

You must have LLVM installed (Mac users, the one provided with XCode is not built with libfuzz support. Good luck)

```
./fuzzing/build.sh  # build fuzzing/fuzzer executable
./fuzzing/run.sh   # starts a fuzzing session
```

## Other tools

If your using the "easy way", just enter into the image with `docker run -it -v $(pwd)/fuzzing:/libddwaf/fuzzing fuzzer bash`

### Rules set

Each fuzzing session use a random rule set generated by `python fuzzing/scripts/build_rules.py` in `./fuzzing/run.sh`

If you want to use a custom rule set, save it as `fuzzing/sample_rules.yml`, comment the line in `run.sh`, and execute `python fuzzing/scripts/build_dict.py`

### See coverage

This script will print a coverage resume of the fuzzing session, and create a `fuzzing/coverage.html` file for detailed coverage informations

```bash
./fuzzing/scripts/show_coverage.sh
```

### Rerun a test : 

```bash
./fuzzing/fuzzer <input_file> --V
```

The `--V` flag will show the PWArgs value
